import sys
import pkg_resources

from style.styled_string_builder import _StyledStringBuilder

try:
    __version__ = pkg_resources.get_distribution('style').version
except Exception:
    __version__ = 'unknown'

_enabled = sys.stdout.isatty()
if '--color' in sys.argv:
    _enabled = True
elif '--no-color' in sys.argv:
    _enabled = False

styled_string_builder = _StyledStringBuilder([], True)
styled_string_builder.enabled = _enabled
styled_string_builder.__version__ = __version__
sys.modules[__name__] = styled_string_builder



import style


class _StyledString(str):

    def __new__(cls, style_list, sep, *objects):
        return super(_StyledString, cls).__new__(cls, sep.join([str(obj) for obj in objects]))

    def __init__(self, style_list, sep, *objects):
        self._style_start = ';'.join([str(s[0]) for s in style_list])
        self._style_end = ';'.join([str(s[1]) for s in style_list])
        self._sep = sep
        self._objects = objects

    def __add__(self, other):
        return self.__str__() + str(other)

    def __str__(self):
        if style._StyledStringBuilder._enabled:
            string = ''
            for i, obj in enumerate(self._objects):
                if i > 0:
                    string += self._sep

                if type(obj) is _StyledString:
                    string += '%s\033[%sm' % (obj, self._style_start)
                else:
                    string += str(obj)
            return '\033[%sm%s\033[%sm' % (self._style_start, string, self._style_end)
        return super(_StyledString, self).__str__()

    def rjust(self, width, fillchar=' '):
        n_chars = width - len(self)
        if n_chars > 0:
            string = str(self)
            return string.rjust(len(string) + n_chars, fillchar)
        return self

    def ljust(self, width, fillchar=' '):
        n_chars = width - len(self)
        if n_chars > 0:
            string = str(self)
            return string.ljust(len(string) + n_chars, fillchar)
        return self



import types

from style.ansi import _styles
from style.styled_string import _StyledString


_module_name = __name__.split('.')[0]


class _StyledStringBuilder(types.ModuleType):
    _enabled = True

    @classmethod
    def enabled(cls):
        return cls._enabled

    @classmethod
    def enabled(cls, value):
        cls._enabled = value

    def __init__(self, style_list, is_root=False):
        super(_StyledStringBuilder, self).__init__(_module_name)
        self._style_list = style_list
        self._is_root = is_root

    def __call__(self, *objects, **kwargs):
        if self._is_root:
            raise TypeError('%r object is not callable' % self.__class__.__bases__[0].__name__)

        sep = kwargs.get('sep', ' ')
        if type(sep) is not str:
            raise TypeError('sep must be None or a string, not %r' % sep.__class__.__name__)

        return _StyledString(self._style_list, sep, *objects)

    def __getattr__(self, attr):
        if attr in _styles:
            if self._is_root:
                new_style_list = self._style_list[:]
                new_style_list.append(_styles[attr])
                return _StyledStringBuilder(new_style_list)
            self._style_list.append(_styles[attr])
            return self
        raise AttributeError('%r object has no attribute %r' % (self.__class__.__name__, attr))
